program galaxy_gen;
// Uses code by Tereguloff

{$APPTYPE CONSOLE}
{$R+}

uses SysUtils, Math;

const
  max_coord = 1000;
  disp_max = 10000;
  disp_levels = 5;
  slim = 1000;
  inf = 100000000;
  kmax = 11;
  mmax = 1010;
  nmax = 1010;

type
  TVector = class
  private
    items_: array of extended; 
    function GetItem(index: integer): extended;
    procedure SetItem(index: integer; x: extended);
  public
    property items[index: integer]: extended read GetItem write SetItem; default;
    constructor Create(n: integer);
    function Size: integer;
  end;
 
  TMatrix = class
  private 
    items_: array of array of extended;
    function GetItem(row, col: integer): extended;
    procedure SetItem(row, col: integer; x: extended);
  public
    property items[row, col: integer]: extended read GetItem write SetItem; default;
    constructor Create(rows, cols: integer);
    function RowCount: integer;
    function ColCount: integer;
  end;
  TVectorArray = array of TVector;
  TMatrixArray = array of TMatrix;
  TVectorArrayPtr = ^TVectorArray;
  TMatrixArrayPtr = ^TMatrixArray;

constructor TVector.Create (n: integer);
begin
  SetLength(items_, n);
end;
function TVector.Size: integer;
begin
  result := high(items_) + 1;
end;
function TVector.GetItem(index: integer): extended;
begin
  result := items_[index];
end;
procedure TVector.SetItem(index: integer; x: extended);
begin
  items_[index] := x;
end;

constructor TMatrix.Create(rows, cols: integer);
begin
  SetLength(items_, rows, cols);
end;
function TMatrix.RowCount: integer;
begin
  result := high(items_) + 1;
end;
function TMatrix.ColCount: integer;
begin
  result := high(items_[0]) + 1;
end;
function TMatrix.GetItem(row, col: integer): extended;
begin
  result := items_[row, col];
end;
procedure TMatrix.SetItem(row, col: integer; x: extended);
begin
  items_[row, col] := x;
end;

function GenerateNormalSelection (var mean, disp: TVector; var corr: TMatrix; count: integer): TMatrix;
var
  v1, v2, s, q: double;

  function getNextNormValue(Idx: integer): double;
  begin
    if (Idx mod 2 = 0) then begin
      repeat
        v1 := 2.0 * Random - 1.0;
        v2 := 2.0 * Random - 1.0;
        s := Sqr(v1) + Sqr(v2);
      until (1.0 - s > 1.0e-6);
      q := Sqrt(-2.0 * Ln(s) / s);
      Result := v1 * q;
    end
    else
      Result := v2 * q;
  end;

var
  a: array of array of double;
  etha: array of double;
  i, j, k, ai, aj, dimensions: integer;
  d: double;

begin
  dimensions := mean.Size;
  assert(dimensions = disp.Size);
  assert(dimensions = corr.RowCount); assert(dimensions = corr.ColCount);
  result := TMatrix.Create(count, dimensions);

  SetLength(a, Dimensions, Dimensions);
  for j := 1 to Dimensions do begin
    aj := j - 1;
    d := Corr[j-1, j-1]*disp[j-1];
    for k := 0 to aj - 1 do
      d := d - Sqr(a[aj, k]);
    d := Sqrt(d);
    for ai := 0 to aj - 1 do
      a[ai, aj] := 0.0;
    for i := j to Dimensions do begin
      ai := i - 1;
      a[ai, aj] := Corr[i-1, j-1]*sqrt(disp[i-1]*disp[j-1]);
      for k := 0 to aj - 1 do
        a[ai, aj] := a[ai, aj] - a[ai, k] * a[aj, k];
      a[ai, aj] := a[ai, aj] / d;
    end;
  end;
  SetLength(etha, Dimensions);
  try
    for i := 0 to Count - 1 do begin
      for j := 0 to Dimensions - 1 do
        etha[j] := getNextNormValue(j);
      for j := 0 to Dimensions - 1 do begin
        Result[i, j] := Mean[j {+ 1}];
        for k := 0 to j do
          Result[i, j] := Result[i, j] + etha[k] * a[j, k];
      end;
    end;
  except
    FreeAndNil(Result);
    raise;
  end;
end;

var 
  tempi, n, i, j, k, l, m, c1, c2, c, scount, pcount, mn: integer;
  tv: TVector;
  mean, disp: array[0..1] of TVector;
  cor, r: TMatrix;
  alpha: double;
  sel: array[0..1] of TMatrix;
  cl: array[0..1] of integer;
  sclass: array[1..mmax] of integer;
  sample: array[1..mmax, 0..kmax - 1] of integer;
  problem: array[1..nmax, 0..kmax - 1] of integer;

begin
//  assignfile (output, 'input.txt'); rewrite (output);

  val(paramstr(1), k, tempi);
  val(paramstr(2), m, tempi);
  val(paramstr(3), n, tempi);
  val(paramstr(4), randseed, tempi);

  for c := 0 to 1 do
  begin
    mean[c] := TVector.Create(k);
    disp[c] := TVector.Create(k);
    cor := TMatrix.Create(k, k);

    for i := 0 to k - 1 do
    begin
      mean[c][i] := 0;
      disp[c][i] := (random(disp_levels) + 1) * disp_max / (disp_levels + 1);
    end;

    for i := 0 to k - 1 do 
    begin
      for j := 0 to k - 1 do 
        cor[i, j] := 0;
      cor[i,i] := 1;
    end;

    sel[c] := GenerateNormalSelection(mean[c], disp[c], cor, m);
    r := TMatrix.Create(k, k);
    tv := TVector.Create(k);

    for c1 := 0 to k - 1 do
      for c2 := c1 + 1 to k - 1 do 
      begin
        for i := 0 to k - 1 do
        begin 
          for j := 0 to k - 1 do
            r[i, j] := 0;
          r[i, i] := 1;
        end;

        alpha := 2*pi*random;
        r[c1, c1] := cos(alpha); r[c1, c2] := -sin(alpha);
        r[c2, c1] := sin(alpha); r[c2, c2] := cos(alpha);
        for i := 0 to m - 1 do
        begin
          for j := 0 to k - 1 do
          begin
            tv[j] := 0;
            for l := 0 to k - 1 do tv[j] := tv[j] + r[j, l]*sel[c][i, l];
          end;
          for j := 0 to k - 1 do sel[c][i, j] := tv[j];
        end;
      end;
    for i := 0 to k - 1 do
    begin
      mean[c][i] := random(max_coord) - max_coord / 2;
      for j := 0 to m - 1 do 
        sel[c][j, i] := sel[c][j, i] + mean[c][i];
    end;
  end;

  writeln(k, ' ', m, ' ', n);

  cl[0] := 0; cl[1] := 0; scount := 0;
  while (cl[0] + cl[1]) < m do
  begin
    inc(scount);
    j := ord((cl[1] < m) and (random > 0.5));
    sclass[scount] := j;
    for i := 0 to k - 1 do sample[scount, i] := round(sel[j][cl[j], i]);
    inc(cl[j]);
  end;

  pcount := 0;
  while (pcount < n / 2) do
  begin
    inc(pcount);
    for j := 0 to k - 1 do problem[pcount, j] := round(random(max_coord) - max_coord / 2);
  end;

  while (pcount < n) do
  begin
    inc(pcount);
    c := round(random);
    for j := 0 to k - 1 do problem[pcount, j] := round(random(round(sqrt(disp[c][j]))) - mean[c][j]);
  end;

  for i := 0 to k - 1 do
  begin
    mn := inf;
    for j := 1 to scount do mn := min(mn, sample[j, i]);
    for j := 1 to pcount do mn := min(mn, problem[j, i]);

    for j := 1 to scount do dec(sample[j, i], mn);
    for j := 1 to pcount do dec(problem[j, i], mn);
  end;
  
  for i := 1 to scount do 
  begin
    write(sclass[i]);
    for j := 0 to k - 1 do write(' ', sample[i, j]);
    writeln;
  end;

  for i := 1 to pcount do
  begin
    for j := 0 to k - 1 do write(problem[i, j], ' ');
    writeln;
  end;

//  closefile (output);
end.
